package com.threerings.jsp.taglib.agent;

import java.io.IOException;
import java.io.StringWriter;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * Contains one or more BrowserCaseTags.  If more than one matches with the
 * same match value, the top one will be evaluated.
 */
public class BrowserTag extends SimpleTagSupport
{

    @Override
    public void doTag()
        throws JspException, IOException
    {
        // invoke the body to trigger child tag matching
        final JspFragment body = getJspBody();
        final StringWriter writer = new StringWriter();
        body.invoke(writer);

        // invoke the body of the best matched child tag
        if (_matchedTag != null) {
            _matchedTag.doBody();
        }
    }

    /**
     * Called by {@link BrowserCaseTag} when a child tag matches the visitor's
     * user-agent.  Only the child tag with the highest matchValue will be
     * evaluated, or the first in sequence if more than one match equally.
     * @param matchValue Integer ranking the closeness of the match. 1 is a
     *     bad match (eg default), 2 matches one variable (eg browser name),
     *     3 matches two variables (eg name and version).  0 is no match.
     * @param matchedTag Instance of the child tag that made the match.
     */
    public void setMatch(int matchValue, BrowserCaseTag matchedTag) {
        if (matchValue < 1) {
            return;
        }
        if (_matchedTag == null || matchValue > _matchValue) {
            _matchValue = matchValue;
            _matchedTag = matchedTag;
        }
    }

    private int _matchValue;
    private BrowserCaseTag _matchedTag;
}
