package com.threerings.jsp.taglib.choose;

import java.io.IOException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * This child tag or one of its siblings will be randomly chosen by the parent
 * {@link RandomChooseTag} and invoked.
 */
public final class RandomBodyTag extends SimpleTagSupport
{
    @Override
    public void doTag ()
        throws JspTagException
    {
        // this must appear within the correct parent tagRandomChooseTag
        RandomChooseTag parentTag = (RandomChooseTag)findAncestorWithClass(this,
            RandomChooseTag.class);
        if (parentTag == null) {
            throw new JspTagException("Random body tags only valid within a random choose tag.");
        }
        parentTag.addBodyTag(this, _weight);
    }

    public void setWeight (int weight)
    {
        _weight = weight;
    }

    /**
     * Invoke the body of this tag.  This will be called by the parent upon selecting this
     * tag as a match.
     */
    public void doBody()
        throws JspException, IOException
    {
        final JspFragment body = getJspBody();
        body.invoke(null);
    }

    protected int _weight = 1;
}
