package com.threerings.jsp.taglib.choose;

import java.io.IOException;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import com.samskivert.util.Randoms;

/**
 * Randomly selects one of the child tags to display
 */
public final class RandomChooseTag extends SimpleTagSupport
{
    @Override
    public void doTag ()
        throws IOException, JspException
    {
        // invoke the body to trigger child tag registration
        final JspFragment body = getJspBody();
        final StringWriter writer = new StringWriter();
        body.invoke(writer);

        // no errors, but no output either
        if (bodyTags.size() == 0) {
            return;
        }

        // select a random child and invoke it's body
        final RandomBodyTag chosenBody = Randoms.RAND.pick(bodyTags, null);
        if (chosenBody != null) {
            chosenBody.doBody();
        }
    }

    /**
     * Child tags will call this to register themselves to be considered for random evaulation
     */
    protected void addBodyTag (RandomBodyTag bodyTag, int weight)
    {
        bodyTags.put(bodyTag, weight);
    }

    /** Map of child body tags to randomly chose from */
    private Map<RandomBodyTag, Integer> bodyTags = new HashMap<RandomBodyTag, Integer>();
}
