package com.threerings.jsp.taglib.choose;

import java.io.IOException;
import java.io.StringWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import com.threerings.jsp.taglib.tracking.DefaultTrackingFilter;

/**
 * Contains one or more {@link VisitorCaseTag}s.  Only the case tag that matches the correct
 * visitor type (new, return) will be displayed.
 */
public class VisitorChooseTag extends SimpleTagSupport
{
    @Override
    public void doTag()
        throws JspException, IOException
    {
        // calculate the visitor type
        HttpServletRequest req = (HttpServletRequest)((PageContext)getJspContext()).getRequest();
        _visitorType = DefaultTrackingFilter.isNewVisitor(req) ? "new" : "return";

        // invoke the body to trigger child tag matching
        final JspFragment body = getJspBody();
        final StringWriter writer = new StringWriter();
        body.invoke(writer);

        // invoke the body of the best matched child tag
        if (_matchedTag != null) {
            _matchedTag.doBody();
        }
    }

    /**
     * Called by {@link VisitorCaseTag} when a child tag matches.  Only the child tag with
     * the highest matchValue will be kept and evaluated later, or the first in sequence
     * if more than one match equally.
     * @param matchValue Integer ranking the closeness of the match.
     * @param matchedTag Instance of the child tag that made the match.
     */
    protected void setMatch(int matchValue, VisitorCaseTag matchedTag) {
        if (matchValue < 1) {
            return;
        }
        if (_matchedTag == null || matchValue > _matchValue) {
            _matchValue = matchValue;
            _matchedTag = matchedTag;
        }
    }

    /**
     * @return Current visitor type
     */
    public String getVisitorType ()
    {
        return _visitorType;
    }

    /** Value of the current highest-ranking match */
    private int _matchValue;

    /** Child tag that made the current highest-ranking match */
    private VisitorCaseTag _matchedTag;

    /** Current type of visitor (new, return) */
    private String _visitorType;
}