package com.threerings.jsp.taglib.form;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;

import com.threerings.jsp.taglib.xhtml.OverrideTag;

/**
 * Displays an HTML input tag of type="checkbox", which preserves its value after form submit.
 */
public class CheckboxTag extends OverrideTag
{
    @Override
    protected String tagName ()
    {
        return "input";
    }

    /**
     * Put aside the selected value for child options to ask for
     */
    @Override
    public void doTag ()
        throws JspTagException
    {
        // select the value of the form field from the request parameters
        final PageContext pageContext = (PageContext) getJspContext();
        final HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();

        // must appear within a Form tag
        final FormTag formTag = (FormTag) findAncestorWithClass(this, FormTag.class);
        if (formTag == null) {
            throw new JspTagException("Checkbox tags must appear within a Form tag.");
        }

        // use the submitted checked state if there is one, otherwise the default
        final String checkedValue;
        if (formTag.isSubmitting()) {
            checkedValue = request.getParameter(_name);
        }
        else {
            checkedValue = _defaultValue;
        }
        if (checkedValue != null && (checkedValue.equals("true")
            || checkedValue.equals("checked") || checkedValue.equals("on"))) {
            _checked = true;
        }

        super.setDynamicAttribute("", "type", "checkbox");
        if (_checked) {
            super.setDynamicAttribute("", "checked", "checked");
        }

        super.doTag();
    }

    /** Optional default selected value */
    public void setDefault (String defaultValue) {
        _defaultValue = defaultValue;
    }
    private String _defaultValue;

    /**
     * Name is required so it can be looked up in the request params.  Also set it as a dynamic
     * attribute so it will be printed in the HTML tag.
     */
    public void setName (String name) {
        _name = name;
        super.setDynamicAttribute("", "name", name);
    }
    private String _name;

    /** Checked value of this tag */
    private boolean _checked = false;
}