package com.threerings.jsp.taglib.form;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;

/**
 * Implementation of ValidatorTag that matches nonzero-length email format field values.
 * Email format is more restrictive than just rfc822; domain is required.
 */
public class EmailValidatorTag extends ValidatorTag
{
    /**
     * Returns true if the fieldValue is not null and has length greater than zero.
     */
    @Override
    protected boolean isValid (String fieldValue)
    {
        // is it non-empty?
        if (fieldValue == null || fieldValue.length() == 0) {
            return false;
        }

        // is it rfc822 compliant?
        try {
            InternetAddress.parse(fieldValue, true);
        }
        catch (final AddressException e) {
            return false;
        }

        // does it match our pattern?
        // eg "name@domain.com" not "name@domain" or "name.com" or "'name' name@domain.com"
        Matcher matcher = EMAIL_PATTERN.matcher(fieldValue.trim());
        if (!matcher.matches()) {
            return false;
        }
        return true;
    }

    /** [one or more non-space chars] @ [one or more non-space chars] . [one or more letters] */
    private static final Pattern EMAIL_PATTERN = Pattern.compile("[^ ]+@[^ ]+\\.[a-zA-Z]+");
}
