package com.threerings.jsp.taglib.form;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;

import org.apache.commons.lang.StringEscapeUtils;

import com.threerings.jsp.taglib.xhtml.OverrideTag;

/**
 * Displays an HTML select tag, with optional default value that child option tags may
 * query for their default selected status.
 * TODO Combine with or replace com.threerings.jsp.taglib.xhtml.InputTag
 */
public class InputTag extends OverrideTag
{
    @Override
    protected String tagName ()
    {
        return "input";
    }

    /**
     * Put aside the selected value for child options to ask for
     */
    @Override
    public void doTag ()
        throws JspTagException
    {
        // select the value of the form field from the request parameters
        final PageContext pageContext = (PageContext) getJspContext();
        final HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();

        // must appear within a Form tag
        final FormTag formTag = (FormTag) findAncestorWithClass(this, FormTag.class);
        if (formTag == null) {
            throw new JspTagException("Input tags must appear within a Form tag.");
        }

        // use the submitted value if there is one
        if (formTag.isSubmitting()) {
            final String submittedValue = request.getParameter(_name);
            super.setDynamicAttribute("", "value", StringEscapeUtils.escapeHtml(submittedValue));
        }
        else if (_defaultValue != null) {
            super.setDynamicAttribute("", "value", StringEscapeUtils.escapeHtml(_defaultValue));
        }

        super.doTag();
    }

    /** Optional default value */
    public void setDefault (String defaultValue) {
        _defaultValue = defaultValue;
    }
    private String _defaultValue;

    /**
     * Name is required so it can be looked up in the request params.  Also set it as a dynamic
     * attribute so it will be printed in the HTML tag.
     */
    public void setName (String name) {
        _name = name;
        super.setDynamicAttribute("", "name", name);
    }
    private String _name;
}