package com.threerings.jsp.taglib.form;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Implementation of ValidatorTag that matches nonzero-length North American phone numbers.
 * Accepted format includes any string which when all non-digits are trimmed is a ten digit
 * number. Examples: (222) 333-4444, 222.333.4444, 222 333 4444. Also matches: asdf 222 asdf 3
 * asdf 3344 asdf 44.
 */
public class PhoneValidatorTag extends ValidatorTag
{
    /**
     * Returns true if the fieldValue is not null and has length greater than zero.
     */
    @Override
    protected boolean isValid (String fieldValue)
    {
        // is it non-empty?
        if (fieldValue == null || fieldValue.length() == 0) {
            return false;
        }

        // strip out all spaces, braces and dashes
        String cleanedValue = CHARACTERS_TO_STRIP.matcher(fieldValue).replaceAll("");

        // is the cleaned value a 10 digit number?
        Matcher matcher = PHONE_PATTERN.matcher(cleanedValue);
        if (!matcher.matches()) {
            return false;
        }
        return true;
    }

    /** Match non-digits */
    private static final Pattern CHARACTERS_TO_STRIP = Pattern.compile("[^0-9]");

    /** 10 digit number */
    private static final Pattern PHONE_PATTERN = Pattern.compile("[0-9]{10}");
}
