package com.threerings.jsp.taglib.form;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.DynamicAttributes;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import org.apache.commons.lang.StringEscapeUtils;

import com.threerings.jsp.taglib.xhtml.AttributeMap;

/**
 * Displays an HTML textarea tag.
 */
public class TextareaTag extends SimpleTagSupport
    implements DynamicAttributes
{
    protected final AttributeMap attributes = new AttributeMap();

    public void setDynamicAttribute (String namespace, String name, Object value)
    {
        if (value != null)
            attributes.put(name, value.toString());
    }

    @Override
    public void doTag ()
        throws JspTagException, IOException, JspException
    {
        final String value;

        // select the value of the form field from the request parameters or the body
        final PageContext pageContext = (PageContext) getJspContext();
        final HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();

        // must appear within a Form tag
        final FormTag formTag = (FormTag) findAncestorWithClass(this, FormTag.class);
        if (formTag == null) {
            throw new JspTagException("Textarea tags must appear within a Form tag.");
        }

        // use the submitted value if there is one
        if (formTag.isSubmitting()) {
            final String submittedValue = request.getParameter(_name);
            value = submittedValue;
        }
        else if (_defaultValue != null) {
            value = _defaultValue;
        }
        else {
            value = getValueFromBody();
        }

        final Writer out = getJspContext().getOut();
        try {
            out.write("<textarea " + attributes.toString() + ">");
            if (value != null && value.length() > 0) {
                out.write(StringEscapeUtils.escapeHtml(value));
            }
            out.write("</textarea>");

        } catch (final Exception e) {
            throw new JspTagException("TextareaTag couldn't produce output", e);
        }
    }

    /**
     * Invoke the tag body and return its contents, or null if body is empty
     */
    private String getValueFromBody ()
        throws IOException, JspException
    {
        final JspFragment body = getJspBody();
        if (body == null) {
            return null;
        }
        final StringWriter bodyWriter = new StringWriter();
        body.invoke(bodyWriter);
        final StringBuffer bodyBuffer = bodyWriter.getBuffer();
        return bodyBuffer.toString();
    }

    /** Optional default value */
    public void setDefault (String defaultValue) {
        _defaultValue = defaultValue;
    }
    private String _defaultValue;

    /**
     * Name is required so it can be looked up in the request params.  Also set it as a dynamic
     * attribute so it will be printed in the HTML tag.
     */
    public void setName (String name) {
        _name = name;
        setDynamicAttribute("", "name", name);
    }
    private String _name;
}