package com.threerings.jsp.taglib.i18n.resources;

import java.io.IOException;
import java.io.StringWriter;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * Contains one or more {@link LocaleTag}s.  Only the locale tag that best matches the current
 * locale settings will be evaluated.
 */
public class ChooseLocaleTag extends SimpleTagSupport
{

    @Override
    public void doTag()
        throws JspException, IOException
    {
        // invoke the body to trigger child tag matching
        final JspFragment body = getJspBody();
        final StringWriter writer = new StringWriter();
        body.invoke(writer);

        // invoke the body of the best matched child tag
        if (_matchedTag != null) {
            _matchedTag.doBody();
        }
    }

    /**
     * Called by {@link LocaleTag} when a child tag matches the current locale.  Only the child
     * tag with the highest matchValue will be kept and evaluated later, or the first in sequence
     * if more than one match equally.
     * @param matchValue Integer ranking the closeness of the match.
     * @param matchedTag Instance of the child tag that made the match.
     */
    public void setMatch(int matchValue, LocaleTag matchedTag) {
        if (matchValue < 1) {
            return;
        }
        if (_matchedTag == null || matchValue > _matchValue) {
            _matchValue = matchValue;
            _matchedTag = matchedTag;
        }
    }

    /** Value of the current highest-ranking match */
    private int _matchValue;

    /** Child tag that made the current highest-ranking match */
    private LocaleTag _matchedTag;
}
