/*
 * Copyright 2007 Three Rings Design, Inc.
 * Copyright 1999,2004 The Apache Software Foundation.
 *
 * This file's implementation is based entirely on code from
 * Jakarta Taglibs's BundleTag, authored by Tim Dawson <tdawson@wamnet.com>
 */

package com.threerings.jsp.taglib.i18n.resources;

import com.threerings.i18n.I18n;

import javax.servlet.jsp.PageContext;


/**
 *  This private class is used by the bundle and message tags for caching the
 *  Gettext I18n instance in the session and request.
 */
class ResourceHelper
{
    private static final String I18N_REQUEST_KEY = ResourceHelper.class.getName() + ".I18n";

    /**
     * Retrieves the I18n instance cached in the page by a previous call to
     * {@link #setI18n(PageContext, I18n, int)}, using {@link PageContext#findAttribute(String)}.
     *
     * This is used by the TextTag to retrieve the cached default I18n instance.
     *
     * @param The page context to search.
     * @return I18n The cached bundle, or null if not found.
     */
    static protected I18n getI18n (PageContext pageContext)
    {
        return (I18n)pageContext.findAttribute(I18N_REQUEST_KEY);
    }

    /**
     * Retrieves the I18n instance cached in the specified scope by a previous
     * call to {@link #setI18n(PageContext, I18n, int).
     *
     * @param pageContext The page context to search
     * @return I18n The cached bundle, or null if not found.
     */
    static protected I18n getI18n (PageContext pageContext, int scope) {
    	return (I18n)pageContext.getAttribute(I18N_REQUEST_KEY, scope);
    }

    /**
     *  Caches the I18n instance in the request using the {@link #I18N_REQUEST_KEY}.
     *
     *  @param pageContext Page context in which the bundle with be cached.
     *  @param i18n I18n instance to cache.
     *  @param scope Scope passed to {@link PageContext#setAttribute(String, Object, int)}
     */
    static protected void setI18n (PageContext pageContext,
                                 I18n i18n,
                                 int scope)
    {
        // put this in the page so that the getMessage tag can get it quickly
        pageContext.setAttribute(I18N_REQUEST_KEY, i18n, scope);
    }

}
