package com.threerings.jsp.taglib.xhtml;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.DynamicAttributes;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * Prints an applet <object> tag.  Dynamic attributes become <param> tags.
 * TODO Support different java versions http://java.sun.com/j2se/1.5.0/docs/guide/plugin/developer_guide/using_tags.html
 * TODO Implement all possible attributes (class, id, title, style, onclick, etc) or create
 *      param sub-tag to pass in dynamic params.
 */
public class AppletTag extends TagSupport
    implements DynamicAttributes
{
    /**
     * Print the opening <object> tag and inner <param> tags.
     */
    @Override
    public int doStartTag ()
        throws JspException
    {
        // Determine the visitor's browser
        final HttpServletRequest req = (HttpServletRequest) pageContext.getRequest();
        final String agent = req.getHeader("User-Agent");
        boolean isIE = false;
        if (agent != null && agent.toLowerCase().contains("msie")) {
            isIE = true;
        }

        try {
            final JspWriter out = pageContext.getOut();
            out.print("\n" + getOpeningTag(isIE));
        }
        catch (IOException e) {
            throw new JspException("GetdownAppletTag could not write to output.");
        }

        // include body between <object> and </object> for visitors without Java
        return EVAL_BODY_INCLUDE;
    }

    /**
     * Print the closing </object> tag.
     */
    @Override
    public int doEndTag ()
        throws JspException
    {
        try {
            final JspWriter out = pageContext.getOut();
            out.print("</object>\n");
        }
        catch (IOException e) {
            throw new JspException("GetdownAppletTag could not write to output.");
        }

        return super.doEndTag();
    }

    /**
     * Dynamic attributes will become <param> tags.
     */
    public void setDynamicAttribute (String namespace, String name, Object value)
    {
        if (value != null) {
            _dynamicParams.put(name, value.toString().trim());
        }
    }

    /** Applet class name is required */
    public void setClassname (String classname) {
        _classname = classname;
    }

    /** Optional archive attribute - name of jar */
    public void setArchive (String archive) {
        _archive = archive;
    }

    /** Width attribute is optional */
    public void setWidth (String width) {
        _width = width;
    }

    /** Height attribute is optional */
    public void setHeight (String height) {
        _height = height;
    }

    /** Height attribute is optional */
    public void setId (String id) {
        _id = id;
    }

    /**
     * Generate and return html for the <object> tag and its <param>s.  Does not include
     * a closing </object> tag.
     */
    private String getOpeningTag (boolean isIE)
    {
        StringBuffer tag = new StringBuffer();

        // object tag has both attributes and <param> tags
        Map<String,String> attributes = new HashMap<String,String>();
        Map<String,String> params = new HashMap<String,String>();

        // for IE, applet class is stored in the params
        if (isIE) {
            attributes.put("classid", IE_CLASSID);
            attributes.put("codebase", IE_CODEBASE);
            params.put("code", _classname);
        }
        // for other browsers, don't put classid since it breaks webkit
        else {
            attributes.put("type", NON_IE_TYPE);
            params.put("code", _classname);
        }

        // konqueror expects archive as a param, so put it in both places
        if (_archive != null) {
            attributes.put("archive", _archive);
            params.put("archive", _archive);
        }

        if (_width != null) {
            attributes.put("width", _width);
        }
        if (_height != null) {
            attributes.put("height", _height);
        }

        if (_id != null) {
            attributes.put("id", _id);
        }

        // build opening object tag and add params
        tag.append("\n<object ");
        for (String attribute : attributes.keySet()) {
            tag.append(attribute + "=\"" + attributes.get(attribute) + "\" ");
        }
        tag.append(">\n");
        for (String param : params.keySet()) {
            tag.append("\t<param name=\"" + param + "\" value=\"" + params.get(param) + "\"/>\n");
        }
        for (String param : _dynamicParams.keySet()) {
            if (_dynamicParams.get(param).trim().length() > 0) {
                tag.append("\t<param name=\"" + param + "\" value=\"" + _dynamicParams.get(param) + "\"/>\n");
            }
        }

        return tag.toString();
    }

    /** dynamic attributes become dynamic <param> tags */
    private final Map<String, String> _dynamicParams = new HashMap<String,String>();

    /** Optional JAR file containing support classes */
    private String _archive;

    /** Optional width */
    private String _width;

    /** Optional height */
    private String _height;

    /** Optional id. */
    private String _id;

    /** Required class, ie com.threerings.getdown.launcher.GetdownApplet */
    private String _classname;

    /** This tag produces applet objects */
    final private String NON_IE_TYPE = "application/x-java-applet;version=1.5";

    /** For IE, this tells it to use the latest installed version of the java plugin */
    final private String IE_CLASSID = "clsid:8AD9C840-044E-11D1-B3E9-00805F499D93";

    /** IE will automatically download this plugin if the visitor does not have it */
    final private String IE_CODEBASE = "http://java.sun.com/update/1.5.0/jinstall-1_5-windows-i586.cab#Version=5,0,0,5";
}
