package com.threerings.jsp.taglib.xhtml;

import java.util.HashMap;
import java.util.Map;

public class AttributeMap extends HashMap<String, String>
{

    public String asString (String name)
    {
        final StringBuilder result = new StringBuilder();
        appendToBuilder(name, get(name), result);
        return result.toString();
    }

    private void appendToBuilder (String name, String found, final StringBuilder result)
    {
        if (found == null)
            return;

        // Quoting logic - attempt to dodge quotation marks used in the value by using the other
        // legal quote.  If both quote types are used, switch to xml attribute quoting.  The latter
        // won't work in prehistoric browsers such as IE, so avoid using both types of quote in
        // attributes for that browser.
        //
        boolean apos = found.contains("'");
        boolean quote = found.contains("\"");

        String value;
        if (apos && quote)
        {
            value = found.replaceAll("'", "&apos;").replaceAll("\"", "&quot;");
        }
        else
        {
            value = found.toString();
        }

        result.append(name);
        result.append("=");
        appendQuote(result, quote);
        result.append(value);
        appendQuote(result, quote);
    }

    private void appendQuote (StringBuilder builder, boolean quote)
    {
        if (quote)
        {
            builder.append("'");
        }
        else
        {
            builder.append("\"");
        }
    }


    @Override
    public String toString ()
    {
        final StringBuilder builder = new StringBuilder();
        for (final Map.Entry<String, String> entry : entrySet()) {
            if (builder.length() > 0)
                builder.append(" ");

            appendToBuilder(entry.getKey(), entry.getValue(), builder);
        }
        return builder.toString();
    }
}
