//
// $Id: $

package com.threerings.jsp.taglib.xhtml;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletRequest;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * Sets a compatible content-type (mime-type) XHTML for the requesting browser.
 */
public class ContentTypeTag extends TagSupport
{
    /**
     * Set the document character encoding. Defaults to UTF-8 (which you should use too!)
     */
    public void setCharset (String charset)
    {
        this.charset = charset;
    }

    /**
     * Set the document character encoding. Defaults to "application/xhtml+xml" for most
     * browser and "text/html" for a few that can't handle xhtml.
     */
    public void setType (String type)
    {
        this.type = type;
    }

    @Override
    public int doEndTag ()
        throws JspException
    {
        final HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
        final HttpServletResponse response = (HttpServletResponse) pageContext.getResponse();

        if (type == null) {
            type = determineContentType(request);
        }

        response.setContentType(type + "; charset=" + charset);

        return super.doEndTag();
    }

    /**
     * If the browser can handle it, return "application/xhtml+xml", otherwise "text/html".
     */
    protected String determineContentType (HttpServletRequest request)
    {
        final String acceptHeader = request.getHeader("Accept");
        boolean xhtml = false;

        if (acceptHeader == null) {
            // This is unlikely. If no header is provided we downgrade to text/html
            xhtml = false;
        } else {
            /*
             * Check if XHTML is preferred over HTML (listed earlier in the ,-delimited list)
             *
             * This is naive, and certainly not perfect, but it works. This implementation
             * doesn't take into account q/level, and may incorrectly match longer mime types,
             * eg, "text/html-weird-mime-type". However, given that we're "downgrading"
             * XHTML to HTML, and don't actually have HTML to provide the requesting client, these
             * defenciencies seem reasonable, and seem to work fine with all modern browsers.
             *
             * If this turns out to be an issue, we can drop in a better Accept header parser.
             *
             * Refer to:
             *  http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html
             */
            final int xhtmlIndex = acceptHeader.indexOf("application/xhtml+xml");
            final int htmlIndex = acceptHeader.indexOf("text/html");
            if (xhtmlIndex != -1 && xhtmlIndex < htmlIndex) {
                xhtml = true;
            }
        }
        return (xhtml) ? "application/xhtml+xml" : "text/html";
    }

    /** Document character encoding. UTF-8 default. */
    private String charset = "utf-8";

    /** Content type to use, defaults to "application/xhtml+xml" or "text/html". */
    private String type;
}