//
// $Id: $

package com.threerings.jsp.taglib.xhtml;

import java.io.Writer;

import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * Prints out a doctype tag of the appropriate type. May be used instead of jsp:output when
 * dynamic doctype switching is needed.
 */
public class DoctypeTag extends SimpleTagSupport
{
    /**
     * Canned doctype to use - may be "xhtml10_strict" or "xhtml10_transitional", defaults to
     * "xhtml10_strict"
     */
    public void setType (String type)
    {
        this.type = type;
    }

    /**
     * Set the root element (defaults to html)
     */
    public void setDoctypeRootElement (String element)
    {
        this.element = element;
    }

    /**
     * Set the doctype-public (optional; overrides output if provided)
     */
    public void setDoctypePublic (String publicDoctype)
    {
        this.publicDoctype = publicDoctype;
    }

    /**
     * Set the doctype-system (optional; overrides output if provided)
     */
    public void setDoctypeSystem (String systemDoctype)
    {
        this.systemDoctype = systemDoctype;
    }

    @Override
    public void doTag ()
        throws JspTagException
    {
        try {
            if (TYPE_XHTML10_STRICT.equals(type)) {
                if (publicDoctype == null) {
                    publicDoctype = "-//W3C//DTD XHTML 1.0 Strict//EN";
                }
                if (systemDoctype == null) {
                    systemDoctype = "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd";
                }
            } else if (TYPE_XHTML10_TRANSITIONAL.equals(type)) {
                if (publicDoctype == null) {
                    publicDoctype = "-//W3C//DTD XHTML 1.0 Transitional//EN";
                }
                if (systemDoctype == null) {
                    systemDoctype = "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd";
                }
            }
            final Writer out = getJspContext().getOut();
            out.write("<!DOCTYPE " + element + " PUBLIC \"" + publicDoctype + "\" \""
                + systemDoctype + "\">\n");
        } catch (final Exception e) {
            throw new JspTagException("DoctypeTag couldn't produce output", e);
        }
    }

    // list of supported canned doctypes
    private static String TYPE_XHTML10_STRICT = "xhtml10_strict";
    private static String TYPE_XHTML10_TRANSITIONAL = "xhtml10_transitional";

    /** Canned doctype to use - may be strict or transitional, defaults to strict. */
    private String type = TYPE_XHTML10_STRICT;

    /** Root element, defaults to html. */
    private String element = "html";

    /** doctype-public, overrides canned doctype-public if provided. */
    private String publicDoctype = null;

    /** doctype-system, overrides canned doctype-system if provided. */
    private String systemDoctype = null;
}