package com.threerings.jsp.taglib.xhtml;

import java.io.StringWriter;
import java.io.Writer;

import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.DynamicAttributes;
import javax.servlet.jsp.tagext.JspFragment;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * Creates an html/xhtml tag with the same name and attributes as are supplied to this tag.  This is useful
 * when you need to use jsp:attribute to supply attributes to an existing tag.  Because there's no obvious
 * way to get the tag name at run-time, this is an ABSTRACT classs and we need to construct a concrete
 * subclass for each tag we want to use.
 */
public abstract class OverrideTag extends SimpleTagSupport
    implements DynamicAttributes
{
    protected final AttributeMap attributes = new AttributeMap();

    public void setDynamicAttribute (String namespace, String name, Object value)
    {
        if (value != null)
            attributes.put(name, value.toString());
    }

    @Override
    public void doTag ()
        throws JspTagException
    {
        final Writer out = getJspContext().getOut();
        try {
            out.write("<" + tagName());
            out.write(" ");
            out.write(attributes.toString());

            // send the body output to a string buffer so we can check its length
            final StringWriter bodyWriter = new StringWriter();
            final JspFragment body = getJspBody();

            // if there is a body
            String bodyString = "";
            if (body != null) {
                body.invoke(bodyWriter);
                final StringBuffer bodyBuffer = bodyWriter.getBuffer();
                bodyString = bodyBuffer.toString();
            }
            if (bodyString.length() != 0 || !selfClosing()) {
                out.write(" >");
                out.write(bodyString);
                out.write("</" + tagName() + ">");
                return;
            }

            // otherwise this is a bodyless tag.
            out.write(" />");

        } catch (final Exception e) {
            throw new JspTagException("OverrideTag couldn't produce output", e);
        }
    }

    /*
     * Return the name of the tag
     */
    protected abstract String tagName ();

    protected boolean selfClosing ()
    {
        return true;
    }
}
