package com.threerings.jsp.taglib.xhtml;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;

/**
 * Work around the internet explorer failure to render PNGs properly.  Under non-ie browsers, this should allow PNGs
 * to render with alpha transparency.
 */
public class PngTag extends OverrideTag
{
    private final Pattern pattern = Pattern.compile("MSIE\\s+(\\d+)\\.(\\d+)\\s*");

    @Override
    protected String tagName ()
    {
        return "img";
    }

    public void setHeight (String height)
    {
        attributes.put("height", height);
    }

    public void setWidth (String width)
    {
        attributes.put("width", width);
    }

    public void setSrc (String src)
    {
        attributes.put("src", src);
    }

    @Override
    public void doTag ()
        throws JspTagException
    {
        final HttpServletRequest req = (HttpServletRequest)((PageContext)getJspContext()).getRequest();
        final String agent = req.getHeader("User-Agent");

        final String src = attributes.get("src");
        if (src != null && src.endsWith(".png")) {
            if (agent != null) {
                if (agent.contains("Win")) {
                    final Matcher matcher = pattern.matcher(agent);
                    if (matcher.find()) {
                        final String major = matcher.group(1);
                        final String minor = matcher.group(2);
                        if (major.equals("6") && minor.equals("0") || major.equals("5")
                            && (minor.equals("0") || minor.equals("5"))) {
                            ieWorkaround();
                        }
                    }
                }
            }
        }

        super.doTag();
    }

    private void ieWorkaround ()
    {
        attributes.put("style", " width: " + attributes.get("width") + "px; " + " height: "
            + attributes.get("height") + "px; "
            + " filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"
            + attributes.get("src") + "', " + " sizingMethod='scale')");

        attributes.put("src", "images/blank.gif");
    }
}
