package com.threerings.jsp.taglib.xhtml;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * Tag to send a redirect from a JSP page.
 */
public class RedirectTag extends TagSupport
{

    private String destination;

    /** If set to true, append the request query string onto the destination url. */
    private boolean preserveQueryString = false;

    public void setDestination (String destination)
    {
        this.destination = destination;
    }

    public void setPreserveQueryString (boolean preserveQueryString)
    {
        this.preserveQueryString = preserveQueryString;
    }

    @Override
    public int doStartTag ()
        throws JspException
    {
        final HttpServletRequest req = (HttpServletRequest) pageContext.getRequest();
        final HttpServletResponse res = (HttpServletResponse)pageContext.getResponse();
        try {

            final String queryString;
            if (preserveQueryString && req.getQueryString() != null) {
                if (destination.contains("?")) {
                    queryString = "&" + req.getQueryString();
                }
                else {
                    queryString = "?" + req.getQueryString();
                }
            }
            else {
                queryString = "";
            }
            res.sendRedirect(destination + queryString);

        } catch (final IOException e) {
            throw new JspException(e);
        }
        return SKIP_PAGE;
    }
}
