package com.threerings.jsp.taglib.xhtml;

import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.DynamicAttributes;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * Implement an html/xhtml script tag that acts just like a <script> tag, but prints a hidden
 * comment at the end of the body, because some browsers can't handle <script /> when no body
 * is supplied.
 */
public class ScriptTag extends TagSupport
    implements DynamicAttributes
{
    protected final AttributeMap attributes = new AttributeMap();

    public void setDynamicAttribute (String namespace, String name, Object value)
    {
        if (value != null) {
            attributes.put(name, value.toString());
        }
    }

    @Override
    public int doStartTag ()
        throws JspTagException
    {
        final JspWriter out = pageContext.getOut();
        try {
            out.print("<script ");
            out.print(attributes);
            out.print(">");
        } catch (final Exception e) {
            throw new JspTagException("ScriptTag couldn't produce output", e);
        }
        return EVAL_BODY_INCLUDE;
    }

    @Override
    public int doEndTag ()
        throws JspTagException
    {
        try {
            final JspWriter out = pageContext.getOut();
            out.print("//</script>");
        } catch (final Exception e) {
            throw new JspTagException("ScriptTag couldn't produce output", e);
        }
        return EVAL_PAGE;
    }
}