package com.threerings.p3p;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletResponse;

/**
 * A simple P3P header filter. Sets the header based on the filter's
 * static configuration. Place "p3p-value" in your FilterConfig
 * to set the desired value.
 *
 * @author landonf@threerings.net
 *
 */
public class SimpleP3PFilter implements Filter {
    /** Filter configuration init parameter for the P3P header value. */
    static final String P3P_INIT_PARAM = "p3p-value";

    /** Name of the P3P header. */
    static final String P3P_HEADER = "P3P";

    public void init (FilterConfig config)
        throws ServletException
    {
        /* Set the default (fallback) locale. */
        this.headerVal = config.getInitParameter(P3P_INIT_PARAM);
        if (this.headerVal == null) {
                throw new ServletException("Missing p3p-value init parameter");
        }
    }

    public void destroy () {
        // Nothing to do here
    }

    public void doFilter (ServletRequest servletRequest, ServletResponse servletResponse, FilterChain chain)
        throws IOException, ServletException
    {
        assert(servletResponse instanceof HttpServletResponse);

        HttpServletResponse response = (HttpServletResponse) servletResponse;

        response.setHeader(P3P_HEADER, headerVal);

        /* Pass the request on. */
        chain.doFilter(servletRequest, servletResponse);
    }

    /** P3P Header Value. */
    private String headerVal;
}
